package com.academiaandroid.formasprimitivas;

//Ejemplo aplicacin Android en el que construiremos formas primitivas en 2D, y objetos grficos en 3D
//con el uso de la API OpenGL ES. Adems se implementan las opciones de iluminacin, definicin de materiales e
//interaccin con el usuario.
//
//academiaandroid.com
//
//by Jos Antonio Gzquez Rodrguez


import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.opengles.GL10;
import android.content.Context;
import android.opengl.GLSurfaceView;
import android.opengl.GLU;
import android.util.Log;
import android.view.MotionEvent;

//Clase que implementa la interfaz Renderer, encargada de 
//realizar las diferentes llamadas para dibujar la imagen, adems de construir la vista
//donde ser representada.
public class GLRender implements GLSurfaceView.Renderer{
	
	//Variable para encender o no la iluminacin en la escena.
	boolean encenderFoco = true; 
	
	//Variables para controlar el tipo de iluminacin y la posicin del foco en la escena.
	private float[] luzAmbiente = {0.2f, 0.2f, 0.2f, 1.0f};
	private float[] luzDifusa = {3.0f, 3.0f, 3.0f, 3.0f};
	private float[] luzPosicion = {1.0f, 1.0f, 2.0f, 1.0f};
	
	//Se declaran las diferentes clases, donde se definen los parmetros de construccin de los objetos grficos.
	private Cuadrado cuadrado;
	private Cubo3D cubo3D;
	private Triangulo triangulo;
	private Piramide3D piramide3D;
	
	//Variable de tipo int que contiene el valor de la forma seleccionada en la Activity principal.
	int flag;
	
	//Variables de tipo float para controlar el ngulo de rotacin de las figuras representadas.
	public float anguloX, anguloY, anguloZ;

	//Se declaran e inicializan dos variables de tipo float para establecer los valores del ngulo y velocidad de 
	//giro del elemento representado.
	private static float angulo = 65;
	private static float velocidadGiroEjeX = -1.5f;
	
	//Constructor donde se inicializan las clases de los diferentes elementos a construir.
	//Recibe como parmetro el elemento seleccionado en la Activity principal.
	public GLRender(int flag, Context context)
	{
		this.cuadrado = new Cuadrado();
		this.cubo3D = new Cubo3D();
		this.triangulo = new Triangulo();
		this.piramide3D = new Piramide3D();
		this.flag = flag;
	}
	
	
	//Mtodo que ser invocado una vez para configurar la vista.
	@Override
	public void onSurfaceCreated(GL10 gl, EGLConfig config) {
		
		//Habilitamos la iluminacin.
		gl.glEnable(GL10.GL_LIGHTING);
		
		//Activamos la primera luz individualmente.
		gl.glEnable(GL10.GL_LIGHT0);
		
		//Introducimos las caractersticas de la luz: luz ambiente, luz difusa y posicin.
		gl.glLightfv(GL10.GL_LIGHT0, GL10.GL_AMBIENT, luzAmbiente,0);
		gl.glLightfv(GL10.GL_LIGHT0, GL10.GL_DIFFUSE, luzDifusa,0);
		gl.glLightfv(GL10.GL_LIGHT0, GL10.GL_POSITION, luzPosicion,0);//Si el ltimo parmetro es 1, la luz se sita en las
		//coordenadas indicadas en los tres primeros, si es 0 se sita en el infinito.
		
		//Por ltimo, especificamos el modo en que OpenGL realizar el sombreado sobre el objeto.
		gl.glShadeModel(GL10.GL_SMOOTH);
	}

	
	//Mtodo que ser invocado si la geometra de la 	
	//vista cambia.
	@Override
	public void onSurfaceChanged(GL10 gl, int width, int height) {		
		//Se comprueba que la altura no sea igual a 0.
		if(height == 0) {                       			
			height = 1;                        			
		}
		//Mtodo que fija la vista recibiendo como parmetros la esquina inferior izquierda del rectngulo 
		//de visualizacin, por defecto (0,0), y los parmetros de ancho y alto.
		gl.glViewport(0, 0, width, height); 
		//Se indica que se trabajar con la matriz PROJECTION.
		gl.glMatrixMode(GL10.GL_PROJECTION);
		//Se carga la identidad de la matriz. 
		gl.glLoadIdentity();		
		//Establece la matriz de proyeccin en perspectiva.
		GLU.gluPerspective(gl, 55.0f, (float)width / (float)height, 0.1f, 100.0f);			
		gl.glMatrixMode(GL10.GL_MODELVIEW);
		gl.glLoadIdentity(); 			
	}
	
	//Mtodo que permite construir los diferentes objetos grficos dentro de la vista definida.
	public void seleccionForma(int flag, GL10 gl)
	{
		//Limpia el buffer para prestablecer los valores de profundidad y color.
		gl.glClear(GL10.GL_COLOR_BUFFER_BIT | GL10.GL_DEPTH_BUFFER_BIT);
		//Al inicializar la variable encenderFoco en true, siempre estar habilitado el foco definido.
		if (encenderFoco) {
	         gl.glEnable(GL10.GL_LIGHTING);
	      } else {
	         gl.glDisable(GL10.GL_LIGHTING);
	      }
        gl.glLoadIdentity();
        //Permite establecer el movimiento de translacin indicando las coordenadas del eje X, Y y Z.
        gl.glTranslatef(0.0f, 0.0f, -5.0f); 
        
         
		 if(flag == 0)//Elemento seleccionado en la Activity principal: Cuadrado 2D.
	        {
			 	//Invocamos el mtodo del objeto cuadrado encargado de establecer los parmetros del grfico a dibujar.
			 	cuadrado.draw(gl);
	        	
	        }else if(flag == 1)//Elemento seleccionado en la Activity principal: Tringulo 2D.
	        {
	        	//Invocamos el mtodo del objeto triangulo encargado de establecer los parmetros del grfico a dibujar.
	        	triangulo.draw(gl);
	        	
	        }else if (flag == 2)//Elemento seleccionado en la Activity principal: Interaccin usuario con objetos 3D.
	        {	
	        	
	        	//Parmetros de rotacin, traslacin y escalado del objeto cubo.
	        	//Permite establecer el movimiento de translacin indicando las coordenadas del eje X, Y y Z.
	        	gl.glTranslatef(-3.0f, 0.0f, -5.0f);
	        	//Permite establecer el tamao de la figura indicando las coordenadas del eje X, Y y Z.
	        	gl.glScalef(0.8f, 0.8f, 0.8f); 
	        		        	
	        	
	        	//Permite establecer el grado de rotacin, indicando adems del ngulo con respecto a un punto,
	        	//las coordenadas del eje X, Y y Z.      	            
	            //Rotacin del objeto cubo a travs del sensor de la pantalla tctil.
	            gl.glRotatef(anguloX, 1.0f, 1.0f, 1.0f);
	            gl.glRotatef(anguloY, 1.0f, 1.0f, 1.0f);
	            gl.glRotatef(anguloZ, 1.0f, 1.0f, 1.0f);       
	            
	            //Invocamos el mtodo del objeto cubo3D encargado de stablecer los parmetros del grfico a dibujar.
	        	cubo3D.draw(gl);
	        	 
	        	//Parmetros de rotacin, traslacin y escalado del objeto pirmide.
	            gl.glScalef(0.8f, 0.8f, 0.8f); 
	        	gl.glTranslatef(-3.0f, 0.0f, -5.0f);
	        	//Rotacin del objeto pirmide a travs del sensor de la pantalla tctil.
	        	gl.glRotatef(anguloX, 1.0f, 1.0f, 1.0f);
	        	gl.glRotatef(anguloY, 1.0f, 1.0f, 1.0f);
	        	gl.glRotatef(anguloZ, 1.0f, 1.0f, 1.0f);
        	
	        	//Invocamos el mtodo del objeto piramide3D encargado de establecer los parmetros del grfico a dibujar.
	        	piramide3D.draw(gl);
	        	
	        }else if(flag == 3)//Elemento seleccionado en la Activity principal: Pirmide 3D.
	        {
	        	gl.glTranslatef(-1.5f, 0.0f, -6.0f);
	        	//Permite establecer el tamao de la figura indicando las coordenadas del eje X, Y y Z.
	        	gl.glScalef(0.8f, 0.8f, 0.8f); 
	        	//Permite establecer el grado de rotacin, indicando adems del ngulo con respecto a un punto,
	        	//las coordenadas del eje X, Y y Z.
	            gl.glRotatef(angulo, 0.1f, 1.0f, -0.1f);
	            //Invocamos el mtodo del objeto piramide3D encargado de establecer los parmetros del grfico a dibujar.
	            piramide3D.draw(gl);
	        	angulo += velocidadGiroEjeX;
	        }
	}

	//Mtodo que ser invocado para cada redibujado de 	la vista. En este ejemplo se realiza la llamada 
	//al mtodo que construir el objeto.
	@Override
	public void onDrawFrame(GL10 gl) {
		seleccionForma(flag, gl);
	}

	public void modificarAngulo(float anguloX, float anguloY, float anguloZ)
	{
		this.anguloX -= anguloX;
		this.anguloY -= anguloY;
		this.anguloZ -= anguloZ; 
	}
	
	//Mtodo que ser invocado por el evento encargado de controlar los cambios que se produzcan en 
	//el sensor de la pantalla tctil. Recibe como argumento la clase MotionEvent.
	public void girarObjetos(MotionEvent event)
	{	
		switch (event.getAction()) {
		case MotionEvent.ACTION_MOVE://Controla los movimientos por la pantalla.
			anguloX = event.getX();
			anguloY = event.getY();	
			Log.d ("DEBUG","ACTION_MOVE"); 
			break;
			
		case MotionEvent.ACTION_DOWN://Controla la pulsacin de la pantalla.
			modificarAngulo(anguloX - event.getX(), anguloY - event.getY(),0.0f);
			anguloX = event.getX();
			anguloY = event.getY();
			Log.d("DEBUG","ACTION_DOWN"); 
			break;
			
		case MotionEvent.ACTION_UP://Controla cuando dejas de pulsar la pantalla.
			modificarAngulo(anguloX + event.getX(), anguloY + event.getY(),0.0f);
			anguloX = event.getX();					
			anguloY = event.getY();
			Log.d("DEBUG","ACTION_UP"); 
			break;

		default:
			break;
		}	
	}
}
